#!/bin/sh

LOGFILE=sysinfo.sh.log

mem='0'
# first try to detect physical memory
# dmidecode is only present on x86/x64 systems
if which dmidecode > /dev/null; then
  mem=$(dmidecode -t memory | awk '$1 == "Size:" && $3 == "GB" { sum += $2 } END { print sum }')
  mem=$((mem * 1073741824))
  # To be backwards compatible with previous versions of dmidecode that produced values in MBs
  if [ "$mem" = "0" ]; then
    mem=$(dmidecode -t memory | awk '$1 == "Size:" && $3 == "MB" { sum += $2 } END { print sum }')
    mem=$((mem * 0x100000))
  fi  
fi

# otherwise use amount of os memory
if [ "$mem" = "0" ]; then
  mem=`sed -nr 's/^MemTotal:[ \t]+([0-9]+).*/\1/p' /proc/meminfo`
  mem=$(($mem * 1024))
fi

swap=`sed -nr 's/^SwapTotal:[ \t]+([0-9]+).*/\1/p' /proc/meminfo`
swap=$(($swap * 1024))
kernel=$(cut -d' ' -f3 /proc/version)

os="Linux"
if [ -e /etc/os-release ]; then
  os=$(grep ^PRETTY_NAME= /etc/os-release | cut -d= -f2)
elif [ -e /etc/lsb-release ]; then
  os=$(grep ^DISTRIB_DESCRIPTION= /etc/lsb-release | cut -d= -f2)
fi
os=${os#\"}
os=${os%\"}
os="$os (kernel $kernel)"

# get libc version
if [ -e /lib/libuClibc-*.so ]; then
  libc=$(ls /lib/libuClibc-*.so | head -n1)
  libc=${libc#*-}
  libc=${libc%.so}
  libc="uClibc $libc"
elif [ -e /lib/libc-*.so ]; then
  libc=$(ls /lib/libc-*.so | head -n1)
  libc=${libc##*-}
  libc=${libc%.so}
  libc="glibc $libc"
elif [ -L /lib/ld-linux.so.* ]; then
  libc=$(readlink /lib/ld-linux.so.*)
  libc=${libc##*-}
  libc=${libc%.so}
  libc="glibc $libc"
fi

if [ -n "$libc" ]; then
  os="$os ($libc)"
fi
if uname -m | grep -q 64; then
  os="$os 64-bit"
else
  os="$os 32-bit"
fi

dmi="/sys/devices/virtual/dmi/id"
if [ -e "$dmi" ]; then
  vendor=$(cat "$dmi/board_vendor")
  product=$(cat "$dmi/board_name")
  fwvendor=$(cat "$dmi/bios_vendor")
  fwver=$(cat "$dmi/bios_version")
elif grep -q poeinfo /proc/cpuinfo; then
  vendor="Exacq Technologies"
  poe_version=$(i2c 30 3)
  npoe=$((4 << (poe_version & 3)))
  product=$(printf "PE%02d-M" $npoe)
  fwver=$(cat /etc/version)
elif which parhandclient > /dev/null; then
  vendor="Axis"
  product=$(parhandclient get root.Brand.ProdFullName - raw 2> /dev/null)
  product=${product#AXIS }
  fwver=$(parhandclient get root.Properties.Firmware.Version - raw 2> /dev/null)
elif [ -e /var/info/sys/brand.xml ]; then
  vendor="Illustra"
  product=$(xml sel -t -v /brand/product/fullname /var/info/sys/brand.xml)
  product=${product#Illustra }
  fwver=$(xml sel -t -v /brand/version/firmware /var/info/sys/brand.xml)
elif [ -e /opt/ad/srv/www/htdocs/data/MODELCACHE.conf ]; then
  vendor="Illustra"
  . /opt/ad/srv/www/htdocs/data/MODELCACHE.conf
  product="$PRODUCTCODE"
  fwver="$FIRMWAREVERSION"
elif [ -e /etc/firmware-version ]; then
  vendor="Vivotek"
  product=$(cut -d- -f1 /etc/firmware-version)
  fwver=$(cut -d- -f3 /etc/firmware-version)
fi

if [ -z "$fwvendor" ]; then
  fwvendor="$vendor"
fi

# get SOC 
if grep -q -i artpec-3 /proc/cpuinfo; then
  soc="Axis artpec-3"
  proc="crisv32"
elif grep -q -i artpec-[45] /proc/cpuinfo; then
  if grep -q -i artpec-4 /proc/cpuinfo; then
    soc="Axis artpec-4"
  else
    soc="Axis artpec-5"
  fi
  proc=$(grep "^cpu model" /proc/cpuinfo | head -n1)
  proc=${proc#*: }
else
  proc=$(grep -m1 ^Processor /proc/cpuinfo)
  proc=${proc#*: }
  if [ -z "$proc" ]; then
    proc=$(grep -m1 "^model name" /proc/cpuinfo)
    proc=${proc#*: }
  fi
  soc=$(grep -m1 ^Hardware /proc/cpuinfo)
  soc=${soc#*: }
  case "$soc" in
    hi3516a) soc="HiSilicon hi3516a"; proc="ARM Cortex-A7";;
    hi3535) soc="HiSilicon hi3535"; proc="ARM Cortex-A9";;
    Axis|Coconut|Condor) soc="Ambarella A5S";;
    Two-Face) soc="Ambarella S2-88"; proc="ARM Cortex-A9";;
    "Ambarella S2L"*) soc="Ambarella S2L"; proc="ARM Cortex-A9";;
    adillustra610) soc="Texas Instruments TI8148"; proc="ARM Cortex-A8";;
    DaVinci*) soc="Texas Instruments $soc";;
    *Xarina*) soc="Sony Xarina";;
  esac
  if [ -n "$soc" ]; then
    proc="$proc ($soc)"
  fi
fi

cat <<EOF
motherboard-vendor=$vendor
motherboard-model=$product
firmware-vendor=$fwvendor
firmware-version=$fwver
memory=$mem
swap=$swap
processor=$proc
os=$os
EOF

# Get versions of exacq's deploy key and base image if found.
deploykey_version=$(grep -oP "^Key Name: \K.*" /versions.txt 2>/dev/null)
if [ $? -eq 0 ] && [ ! -z "$deploykey_version" ]; then
    echo "deploykey-version=$deploykey_version"
fi
deploykey_jobnumber=$(grep -oP "^Job: \K[0-9]*" /versions.txt 2>/dev/null)
if [ $? -eq 0 ] && [ ! -z "$deploykey_jobnumber" ]; then
    echo "deploykey-jobnumber=$deploykey_jobnumber"
fi
deploykey_branchname=$(grep -oP "^Branch: \K.*" /versions.txt 2>/dev/null)
if [ $? -eq 0 ] && [ ! -z "$deploykey_branchname" ]; then
    echo "deploykey-branchname=$deploykey_branchname"
fi
# Note that the release notes pathname varies between OPS and RETAIL keys.
baseimage_version=$(grep -oP "Linux-1804/Release_Notes.txt:\K.*" /versions.txt 2>/dev/null)
if [ $? -eq 0 ] && [ ! -z "$baseimage_version" ]; then
    echo "baseimage-version=$baseimage_version"
fi


for edvrmountpt in $(ls /mnt/edvr); do
  # In 18.04, the exacq image configures all RAID and non-RAID drives with x-systemd.automount
  # which permits server to startup without waiting for mounts to complete. Hence this may
  # execute before recording drives mounted, and therefore may not see expected partitions and
  # therefore indicate false positive "needs prep". The easy cure for this is to just try
  # to access the output file from PSFPI's read-write test. Swallow any errors, because this
  # is just meant to prod automount just in case.
  cat "/mnt/edvr/$edvrmountpt/psfpi/test.bin" > /dev/null 2>&1
done


rm -f $LOGFILE
echo "=============================================" >> $LOGFILE


###################################################################################################
###################################################################################################
#                         PHYSICAL DISK LOCATION DETECTION
#
# See https://teams.microsoft.com/l/channel/19%3A7aae6d1c51da4286b68cea8bbd4ae7c4%40thread.tacv2/tab%3A%3A126f6312-ab59-4a4f-9358-646e0dd72af8?groupId=3c4070f6-47e4-4250-ac38-73f115582044&tenantId=a1f1e214-7ded-45b6-81a1-9e8ae3459641
###################################################################################################
###################################################################################################

# This is executed in Edge platforms, so can't use bash-isms below.

if which storcli > /dev/null 2>&1; then
    # If this is a RAID system, want to report RAID volumes which are partitioned & mounted, so SysmgmtPI
    # can report their mountpoints for evClient to allow double-clicking a RAID disk on the Storage HW tab
    # and jumping to its volume's drive on the Drives tab. This also allows SysmgmtPI to detect an
    # unmounted volume and permit prepping it, like if evServer restarts during volume create and wants
    # to allow user to click and resume raidprep. And the raidprep script is smart enough to partition,
    # format, and/or mount as needed.
    #
    # Reuse existing raidprep script with "-i" so it will just output info resembling:
    # raidprep.sh: /c0/v1 matches /dev/sdb - unpartitioned so prep it.     <=== omit this
    # raidprep.sh: /c0/v2 matches /dev/sdc - partitioned so skip.          <=== include this
    #
    # Output will resemble:
    # raid-volume=/c0/v1
    # raid-fspath=/mnt/edvr/1     <=== only report first mountpoint found, never empty
    /usr/local/exacq/server/scripts/raidprep.sh -i | grep "matches" | grep " \- partitioned" | while read line; do
        VOLNAME="$(echo "$line" | cut -d' ' -f2)"    # like /c0/v0
        DEVNAME="$(echo "$line" | cut -d' ' -f4)"    # like /dev/sda
        RAID_PART_INFO="$(lsblk -no TYPE,KNAME,MOUNTPOINT $DEVNAME 2>&1 | grep -m 1 part)"  # only look at first partition & mount
        RAID_PART_INFO="$(echo $RAID_PART_INFO | sed -e "s/[[:space:]]\+/ /g")"    # prune all repeated whitespace
        MOUNTPT="$(echo "$RAID_PART_INFO" | cut -d' ' -f3)"    # like /mnt/edvr/1
        [ -z "$MOUNTPT" ] && continue
        echo "raid-volume=$VOLNAME"
        echo "raid-fspath=$MOUNTPT/"     # psfpi expects trailing slash
    done
    exit 0
fi


has_lsi_hba=0
if which lspci > /dev/null && lspci | grep -q 'SAS3008'; then
    has_lsi_hba=1
    echo "Detected LSI 9300."
else
    echo "No LSI 9300 detected."
fi


is_el_series=0
grep -q "<family value=\"elp_series\" />" /usr/local/exacq/server/evsysteminfo.xml && is_el_series=1
grep -q "<family value=\"elpr_series\" />" /usr/local/exacq/server/evsysteminfo.xml && is_el_series=1


if [ "$product" = "C236M WS" ] || [ "$product" = "H110E1" ] || [ "$product" = "IMB-X1313" ] || [ "$product" = "IMB-1313" ] || [ "$product" = "H410E1" ] || [ "$product" = "H410E1T" ]; then
    # Historically, this returned the OS partition devname like /dev/sda1 or /dev/nvme0p1, but starting with 20.04
    # it instead returns a symlink like /dev/disk/by-uuid/d653a46b-064d-400b-b2bd-b5f03c23cba4 because fstab
    # uses UUIDs instead of devnames. So if not a short devname, then must translate UUID to devname.
    # Unfortunately have to use grep here because this script doesn't request bash.

    # On BBE system, the OS drive's first partition is always mounted as EFI. Will resemble "sdX"
    osdrive="$(lsblk -ln -o PKNAME,NAME,FSTYPE,MOUNTPOINT | grep /boot/efi | cut -d " " -f1)"
    echo "OS drive = $osdrive"

    if [ "$product" = "H110E1" ]; then    # I don't want to repeat this for every disk in below loop....
        # And need to check system serial number to determine whether it's an older system with DVD/empty
        # in SATA-10, or in SATA-2. The changeover was 2019 week 43 (2019-10-24).
        # Use cut to extract out of <SerialNumber value="ER1943035523" />
        # Note this is irrelevant for X1313, but less code than adding another branch.
        system_serial_num="$(grep "SerialNumber" /usr/local/exacq/server/evsysteminfo.xml | cut -d\" -f2)"
        # Note this is /bin/sh not /bin/bash so have to use weird POSIX syntax for lexicographic compare.
        if expr "$system_serial_num" \< "ER1943035523" >/dev/null; then    # compare to first "newer" system serial
            dvd_slot_num=10
            echo "System serial = $system_serial_num (older than 2019 week 43, 2019-10-24)."
            echo "Slot $dvd_slot_num is DVD or empty."
        else
            dvd_slot_num=2
            echo "System serial = $system_serial_num (newer than 2019 week 43, 2019-10-24)."
            echo "Slot $dvd_slot_num is DVD or empty."
        fi

        # E1 mobo always reports 4 SATA controllers in order - 4-port with OS drive, then 3 x 2-port for data disks.
        # Can't hardcode PCI paths because they can shift, so have to re-acquire. Will resemble "00:17.0".
        h110_ctlr_os="$(lspci -n | grep "0106: " | sed -n '1 p' | cut -d" " -f1)"
        h110_ctlr_data1="$(lspci -n | grep "0106: " | sed -n '2 p' | cut -d" " -f1)"
        h110_ctlr_data2="$(lspci -n | grep "0106: " | sed -n '3 p' | cut -d" " -f1)"
        h110_ctlr_data3="$(lspci -n | grep "0106: " | sed -n '4 p' | cut -d" " -f1)"
        echo "Detected H110E1 SATA ctlrs - OS [$h110_ctlr_os] DATA1 [$h110_ctlr_data1] DATA2 [$h110_ctlr_data2] DATA3 [$h110_ctlr_data3]"
    fi

    # LSI 9300 is SAS HBA, so have to use SAS PHY #s, versus ATA #s for mobo-attached SATA disks.
    # NOTE THAT udev RULES MAY CHANGE BETWEEN OS VERSIONS!
    for devpath in /dev/disk/by-path/pci-????:??:??.?-sas-phy?-lun-0 /dev/disk/by-path/pci-????:??:??.?-ata-? ; do
        echo "#####################################################################"
        # Glob itself is returned if no matches found, so catch.
        [ "$devpath" = "/dev/disk/by-path/pci-????:??:??.?-sas-phy?-lun-0" ] && continue
        [ "$devpath" = "/dev/disk/by-path/pci-????:??:??.?-ata-?" ] && continue

        dev=$(basename $(readlink $devpath))    # convert from ../../sdX
        serial=$(smartctl -i /dev/$dev | grep "Serial Number:" | sed 's/Serial Number://' | awk '{ sub (/^[ \t]+/, ""); print}' )
        [ "$dev" = "sr?" ] && echo "Skipping DVD." && continue

        # Unfortunately not bash so can't regex, but at least can use char positions below.
        ctlr_pcipath=$(basename $devpath | cut -c10-16)   # extract the 00:1f.2 part for the controller's PCI location
        devpath=$(basename $devpath | cut -c18-)          # shorten to the ata-6 or sas-phy1-lun-0 part
        ctlr_pcivendor=$(lspci -n | grep $ctlr_pcipath | cut -d " " -f3 | cut -d ":" -f1)  # controller's PCI vendor ID
        ctlr_pcidevice=$(lspci -n | grep $ctlr_pcipath | cut -d " " -f3 | cut -d ":" -f2)  # controller's PCI device ID

        non_raid_port="-1"   # default to unrecognized or non-data disk
        if echo "$devpath" | grep -q "sas"; then
            # sas is 9300 which normally means C236, but did ship with some E1 to compensate for an old evServer bug.
            # PHY num always points to the same controller port, regardless of anything else.
            phynum=$(echo $devpath | cut -d "-" -f2 | cut -c4-)    # extract from sas-phy1-lun-0
            echo "Found $devpath => $dev => PHY $phynum => $serial"
            # Convert PHY from 0-based to 1-based to use as index with cut cmd in order to get 1-based value from array.
            phynum=$((phynum + 1))
            non_raid_port=$(echo 43128756 | cut -c$phynum)   # not bash, so no array support, so be clever
            [ -z "$non_raid_port" ] && non_raid_port="-1"    # phynum not found in array returns empty string, so re-default
        elif [ "$product" = "C236M WS" ]; then
            # C236 mobo ports show up as ATA, which likewise always seems to point to same SATA port on mobo.
            atanum=$(echo $devpath | cut -c5- )   # extract from ata-6
            echo "Found $devpath => $dev => ATA $atanum => $serial"
            # Recognize ATA nums 2, 3, 4, or 5, and use as index with cut cmd in order to get 1-based value from array.
            non_raid_port=$(echo 01234 | cut -c$atanum)      # not bash, so no array support, so be clever
            [ -z "$non_raid_port" ] && non_raid_port="-1"    # atanum not found in array returns empty string, so re-default
        elif [ "$product" = "H110E1" ]; then
            atanum=$(echo $devpath | cut -c5- )   # extract from ata-6
            if [ $is_el_series -eq 1 ]; then
                # Unfortunately this deviates. It's SSD, right data, left data, DVD/empty.
                echo "Found $devpath => $dev => ATA $atanum => $serial (EL-Series)"
                [ $atanum -eq 2 ] && non_raid_port="2"
                [ $atanum -eq 3 ] && non_raid_port="1"
                [ $atanum -eq 4 ] && non_raid_port="1"    # just in case
            elif [ "$dvd_slot_num" = "2" ]; then
                [ "$ctlr_pcipath:$atanum" = "$h110_ctlr_os:1" ]    && bus="OS"  # this is OS
                [ "$ctlr_pcipath:$atanum" = "$h110_ctlr_os:2" ]    && bus="OS"  # this is DVD or empty
                [ "$ctlr_pcipath:$atanum" = "$h110_ctlr_os:3" ]    && non_raid_port="1" && bus="OS"
                [ "$ctlr_pcipath:$atanum" = "$h110_ctlr_os:4" ]    && non_raid_port="2" && bus="OS"
                [ "$ctlr_pcipath:$atanum" = "$h110_ctlr_data1:1" ] && non_raid_port="3" && bus="DATA1"
                [ "$ctlr_pcipath:$atanum" = "$h110_ctlr_data1:2" ] && non_raid_port="4" && bus="DATA1"
                [ "$ctlr_pcipath:$atanum" = "$h110_ctlr_data2:1" ] && non_raid_port="5" && bus="DATA2"
                [ "$ctlr_pcipath:$atanum" = "$h110_ctlr_data2:2" ] && non_raid_port="6" && bus="DATA2"
                [ "$ctlr_pcipath:$atanum" = "$h110_ctlr_data3:1" ] && non_raid_port="7" && bus="DATA3"
                [ "$ctlr_pcipath:$atanum" = "$h110_ctlr_data3:2" ] && non_raid_port="8" && bus="DATA3"
                echo "Found $devpath => $dev => $serial (bus $bus)"
            else
                [ "$ctlr_pcipath:$atanum" = "$h110_ctlr_os:1" ]    && bus="OS"  # this is OS
                [ "$ctlr_pcipath:$atanum" = "$h110_ctlr_os:2" ]    && non_raid_port="1" && bus="OS"
                [ "$ctlr_pcipath:$atanum" = "$h110_ctlr_os:3" ]    && non_raid_port="2" && bus="OS"
                [ "$ctlr_pcipath:$atanum" = "$h110_ctlr_os:4" ]    && non_raid_port="3" && bus="OS"
                [ "$ctlr_pcipath:$atanum" = "$h110_ctlr_data1:1" ] && non_raid_port="4" && bus="DATA1"
                [ "$ctlr_pcipath:$atanum" = "$h110_ctlr_data1:2" ] && non_raid_port="5" && bus="DATA1"
                [ "$ctlr_pcipath:$atanum" = "$h110_ctlr_data2:1" ] && non_raid_port="6" && bus="DATA2"
                [ "$ctlr_pcipath:$atanum" = "$h110_ctlr_data2:2" ] && non_raid_port="7" && bus="DATA2"
                [ "$ctlr_pcipath:$atanum" = "$h110_ctlr_data3:1" ] && non_raid_port="8" && bus="DATA3"
                [ "$ctlr_pcipath:$atanum" = "$h110_ctlr_data3:2" ] && bus="DATA3"  # this is DVD or empty
                echo "Found $devpath => $dev => $serial (bus $bus)"
            fi
        elif [ "$product" = "IMB-X1313" ] || [ "$product" = "IMB-1313" ]; then
            # X1313 is custom board with W480E chipset for Xeon and ECC, but due to supply chain we also shipped stock 1313 with Q470 chipset.
            # These mobos have a 6-cluster for data disks 1-4 with last 2 ports disabled because NVMe, then 2-cluster.
            # Then if fixed-disk chassis with >= 5 disks, or if removable-disk chassis, we add m.2 expander with 4-cluster
            # and use last 2 ports for disks 7 & 8. Disk enumeration order:
            #     -- CTLR3 m.2 expander first if present <== SCSI port 0.
            #     -- CTLR2 2-cluster on mobo             <== SCSI port 1 (or shifted if no expander)
            #     -- NVMe                                <== SCSI port 2 (or shifted if no expander)
            #     -- CTLR1 6-cluster on mobo             <== SCSI port 3 (or shifted if no expander)
            # Also, WinSvr swaps SCSI port for the NVMe and 6-cluster for unknown reasons! Since this is all such
            # a hot mess, use the owning IDE controller's PCI vendor/device.
            CTLR1="8086:06d2"; CTLR2="1b21:0612"; CTLR3="1b21:1064"
            atanum=$(echo $devpath | cut -c5- )   # extract from ata-6
            [ "$ctlr_pcivendor:$ctlr_pcidevice:$atanum" = "$CTLR1:1" ] && non_raid_port="1" && bus="OS"
            [ "$ctlr_pcivendor:$ctlr_pcidevice:$atanum" = "$CTLR1:2" ] && non_raid_port="2" && bus="OS"
            [ "$ctlr_pcivendor:$ctlr_pcidevice:$atanum" = "$CTLR1:3" ] && non_raid_port="3" && bus="OS"
            [ "$ctlr_pcivendor:$ctlr_pcidevice:$atanum" = "$CTLR1:4" ] && non_raid_port="4" && bus="OS"
            [ "$ctlr_pcivendor:$ctlr_pcidevice:$atanum" = "$CTLR2:1" ] && non_raid_port="5" && bus="DATA"
            [ "$ctlr_pcivendor:$ctlr_pcidevice:$atanum" = "$CTLR2:2" ] && non_raid_port="6" && bus="DATA"
            [ "$ctlr_pcivendor:$ctlr_pcidevice:$atanum" = "$CTLR3:3" ] && non_raid_port="7" && bus="EXPANDER"
            [ "$ctlr_pcivendor:$ctlr_pcidevice:$atanum" = "$CTLR3:4" ] && non_raid_port="8" && bus="EXPANDER"
            echo "Found $devpath => $dev => $serial (bus $bus)"
        elif [ "$product" = "H410E1" ] || [ "$product" = "H410E1T" ]; then
            # These mobos have a 4-cluster for data disks 1-4.
            # Then if fixed-disk chassis with >= 5 disks, or if removable-disk chassis, we add m.2 expander
            # with 4-cluster for data disks 5-8. Using same technique as X1313, with PCI info from ctlr.
            CTLR1="8086:a382"; CTLR2="1b21:1064"
            atanum=$(echo $devpath | cut -c5- )   # extract from ata-6
            [ "$ctlr_pcivendor:$ctlr_pcidevice:$atanum" = "$CTLR1:1" ] && non_raid_port="1" && bus="OS"
            [ "$ctlr_pcivendor:$ctlr_pcidevice:$atanum" = "$CTLR1:2" ] && non_raid_port="2" && bus="OS"
            [ "$ctlr_pcivendor:$ctlr_pcidevice:$atanum" = "$CTLR1:3" ] && non_raid_port="3" && bus="OS"
            [ "$ctlr_pcivendor:$ctlr_pcidevice:$atanum" = "$CTLR1:4" ] && non_raid_port="4" && bus="OS"
            [ "$ctlr_pcivendor:$ctlr_pcidevice:$atanum" = "$CTLR2:1" ] && non_raid_port="5" && bus="EXPANDER"
            [ "$ctlr_pcivendor:$ctlr_pcidevice:$atanum" = "$CTLR2:2" ] && non_raid_port="6" && bus="EXPANDER"
            [ "$ctlr_pcivendor:$ctlr_pcidevice:$atanum" = "$CTLR2:3" ] && non_raid_port="7" && bus="EXPANDER"
            [ "$ctlr_pcivendor:$ctlr_pcidevice:$atanum" = "$CTLR2:4" ] && non_raid_port="8" && bus="EXPANDER"
            echo "Found $devpath => $dev => $serial (bus $bus)"
        fi

        if [ "$non_raid_port" = "-1" ] || [ "$osdrive" = "$dev" ]; then
            # If OS drive, should not match any of the above rules, but just in case, check for ==$osdrive.
            echo "Skipping unrecognized or OS disk [$dev] $serial at $devpath on ctlr $ctlr_pcipath (PCI $ctlr_pcivendor:$ctlr_pcidevice)."
            continue
        fi

        numparts="$(ls /dev/$dev? 2>/dev/null | wc -l)"
        [ $numparts = "1" ] && fspath="$(lsblk -n -o MOUNTPOINT /dev/${dev}1)/" || fspath=""  # psfpi expects trailing /
        [ $numparts = "1" ] && fstype="$(lsblk -n -o FSTYPE /dev/${dev}1)" || fstype=""

        echo "drive-index=$non_raid_port" | tee -a $LOGFILE    # stdout AND append logfile
        echo "drive-serial=$serial" | tee -a $LOGFILE
        echo "drive-name=$dev" | tee -a $LOGFILE
        echo "drive-partitions=$numparts" | tee -a $LOGFILE
        echo "drive-fspath=$fspath" | tee -a $LOGFILE
        echo "drive-fstype=$fstype" | tee -a $LOGFILE
        if [ ! "$fstype" = "ext4" ] || ! echo "$fspath" | grep -q "/mnt/edvr" || [ $numparts -ne 1 ]; then
            echo "drive-needs-prep=1" | tee -a $LOGFILE
        fi
    done

    exit 0
fi


for dev in $(ls /sys/block/ | grep ^sd); do
    # This is executed in Edge platforms, so not necessarily bash, hence may not be able to refactor into function.
    echo "#####################################################################"

    # Refer to https://dump.4network.org/2016/10/06/match-linux-scsi-devices-sdx-to-virtual-disks-in-vmware/
    # The scsi_device dir has a single filename like "1:2:3:4" which means
    # <HOST_ADAPTER_ID>:<SCSI_CHANNEL>:<DEVICE_ID>:<LUN>
    scsi_device_path="/sys/block/$dev/device/scsi_device"
    scsi_device="$(ls $scsi_device_path)"  # like "1:2:3:4"
    echo "Querying disk at $scsi_device_path - $scsi_device"

    non_raid_port="-1"   # default to unrecognized or non-data disk

    # #15756  - For SIIG, scsi ids look likst scsi@B:D1.D2.D3
    # where B: busid, and Dx are device id fields. B is unique and Dx are all 0's 
    #
    # For LSI, busid is always 0. but D2 field is unique. 
    # Offsetting the d2 value on LSI by 99, that gives indices in the range 99->107. 
    # Also offset the 'b' by hundred as well. This ensures that the OS drive falls out of range and is not displayed in the gui. 
    # So final formula for lsi hba is index = (b * 100) + d2 + 99
    # Use this value in hwmconfig.xml drive ordering
    # Note there's a corresponding stanza in hwmconfig for Linux just for Q87 mobo.
    #
    # This formula ensures existing SIIG ordering is unchanged because all the device ids are 0's. 
    # So if scsiid for a SIIG device is 2:0.0.0, index will still be 2. 
    # Also ensures that on a system with LSI, the unique index does not collide with SIIG index except index 0.
    #
    # Note that hwmconfig does not have a rule for the SSD's drive-index value.
    rm -f /tmp/non_raid_port*.tmp
    echo "$scsi_device" | {
        # Since this script is run on Edge without bash, not relying on process substitution.
        # Go old school tmp file instead, to get values out of subshell.
        IFS=: read b d1 d2 d3
        echo "$b"  >> /tmp/non_raid_port1.tmp
        echo "$d1" >> /tmp/non_raid_port2.tmp
        echo "$d2" >> /tmp/non_raid_port3.tmp
        echo "$d3" >> /tmp/non_raid_port4.tmp
    }
    b=$(cat /tmp/non_raid_port1.tmp)
    d1=$(cat /tmp/non_raid_port2.tmp)
    d2=$(cat /tmp/non_raid_port3.tmp)
    d3=$(cat /tmp/non_raid_port4.tmp)
    [ $has_lsi_hba -eq 0 ] && non_raid_port="$b" || non_raid_port="$((b * 100 + d2 + 99))"

    [ "$non_raid_port" = "-1" ] &&
        echo "Skipping unrecognized or system disk $dev at $scsi_device" &&
        continue
    echo "drive-index=$non_raid_port"
    echo "drive-index=$non_raid_port" >> $LOGFILE

    # FYI, if C236 with disks on mobo, udevadm info normally returns serial # in
    # ID_SCSI_SERIAL and/or ID_SERIAL_* fields. But with LSI 9300, they DO NOT return
    # serial. They return other strings. The serial instead appears in SCSI_IDENT_SERIAL
    # but then that means looking for 3-4 parm names, and too complicated. So just
    # leverage smartctl instead:    Serial Number:    WD-WMAY03230613
    # Remember, this isn't bash, so can't depend on any bashisms....
    serial=$(smartctl -i /dev/$dev | grep "Serial Number:" | sed 's/Serial Number://' | awk '{ sub(/^[ \t]+/, ""); print}' )
    echo "drive-serial=$serial"
    echo "drive-serial=$serial" >> $LOGFILE

    removable=$(cat /sys/block/$dev/removable)
    fspath=
    fstype=
    system=0
    numparts=0
    while read x1 x2 x3 x4; do
        if [ "/dev/$dev" = "${x1%%[0-9]*}" ]; then
            # psfpi expects trailing /
            fspath=${x2%/}/
            fstype=$x3
            if [ "$fspath" = "/" -o "$fspath" = "/mnt/export/" -o "$fstype" = "swap" ]; then
                system=1
            fi
            numparts=$((numparts + 1))
        fi
    done < /etc/mtab

    # Should have been caught above, but just in case.
    [ "$system" = "1" ] && echo "Skipping unrecognized or system disk." && continue
    [ "$removable" = "1" ] && echo "Skipping removable disk." && continue

    echo "drive-name=$dev" | tee -a $LOGFILE    # stdout AND append logfile
    echo "drive-partitions=$numparts" | tee -a $LOGFILE
    echo "drive-removable=$removable" | tee -a $LOGFILE
    echo "drive-fspath=$fspath" | tee -a $LOGFILE
    echo "drive-fstype=$fstype" | tee -a $LOGFILE
    if [ ! "$fstype" = "ext4" ] || ! echo "$fspath" | grep -q "/mnt/edvr" || [ $numparts -ne 1 ]; then
        echo "drive-needs-prep=1" | tee -a $LOGFILE
    fi
done
